/*
 * Copyright (C) 2010-2017 Univention GmbH <http://www.univention.de/>
 * by order of
 *     Open-Xchange (http://www.open-xchange.com)
 *
 * http://www.univention.de/
 *
 * All rights reserved.
 *
 * The source code of this program is made available
 * under the terms of the GNU Affero General Public License version 3
 * (GNU AGPL V3) as published by the Free Software Foundation.
 *
 * Binary versions of this program provided by Univention to you as
 * well as other copyrighted, protected or trademarked materials like
 * Logos, graphics, fonts, specific documentations and configurations,
 * cryptographic keys etc. are subject to a license agreement between
 * you and Univention and not subject to the GNU AGPL V3.
 *
 * In the case you use this program under the terms of the GNU AGPL V3,
 * the program is provided in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License with the Debian GNU/Linux or Univention distribution in file
 * /usr/share/common-licenses/AGPL-3; if not, see
 * <http://www.gnu.org/licenses/>.
 */
/*global define console*/

define([
	"dojo/_base/declare",
	"dojo/_base/lang",
	"dojo/_base/array",
	"dojo/dom-class",
	"dojo/topic",
	"dojo/Deferred",
	"dijit/Dialog",
	"umc/dialog",
	"umc/widgets/Wizard",
	"umc/widgets/Module",
	"umc/widgets/Form",
	"umc/widgets/TextBox",
	"umc/widgets/PasswordBox",
	"umc/widgets/CheckBox",
	"umc/widgets/ComboBox",
	"umc/widgets/Text",
	"umc/i18n!umc/modules/oxldb"
], function(declare, lang, array, domClass, topic, Deferred, Dialog, dialog, Wizard, Module, Form, TextBox, PasswordBox, CheckBox, ComboBox, Text, _) {
	var oxldb = declare([ Wizard ], {

		standbyOpacity: 1.00,

		_dummyPassword: '!*****DUMMY***PASSWORD*****!',
		_credentialsValid: null,
		_licensekey: '',
		_currentLicensekey: null,
		_ldapBase: null,
		_serverRole: 'domaincontroller_master',

		constructor: function() {
			this.pages = [{
				name: 'pageAccount',
				headerText: _('Step 1 of 2: Open-Xchange Customer Id'),
				helpText: _('<p>This module will assist you in configuring your Open-Xchange Customer Id and license key.</p>'),
				widgets: [{
					type: TextBox,
					name: 'username',
					label: _('Customer Id'),
					required: true
				}, {
					type: PasswordBox,
					name: 'password',
					label: _('Password'),
					required: true
				}, {
					type: Text,
					name: 'passwordlost',
					content: _('<a href="https://servicecenter.open-xchange.com/Servicecenter/" target="_blank">Passwort zurücksetzen</a></p>')
				}, {
					type: Text,
					name: 'waitmsg',
					content: _('Retrieving license data for selected LDB account... please wait...')
				}, {
					type: Text,
					name: 'savewaitmsg',
					content: _('Updating system configuration ... please wait...')
				}  ],
				layout: [{
					label: _('OX License Management'),
					name: 'accountPane',
					layout: [
						['username'],
						['password', 'passwordlost']
					]
				}]
			}, {
				name: 'pageLicense',
				headerText: _('Step 2 of 2: Open-Xchange License Key'),
				helpText: '',
				widgets: [{
					type: Text,
					name: 'txt1',
					content: _('<p>Your given Customer Id has been tested successfully.</p>')
				}, {
					type: Text,
					name: 'txt2',
					content: _('<p>As second step a Open-Xchange license key has to be entered for this system. The license key will be automatically inserted to the relevant configuration files.</p>')
				},{
					type: Text,
					name: 'txt4',
					content: '<p>&nbsp;</p>'
				}, {
					type: TextBox,
					name: 'licensekey',
					label: _('Open-Xchange License Key'),
					required: true,
					regExp: '(^OX6?-(SE(4UCS)?|APPS|DOC)(-.*?)?-(MK|SUB)-|^[0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12}$)',
					style: 'width:35em;'
				}],
				layout: [{
					label: _('Open-Xchange License Key'),
					name: 'licensePane',
					layout: [
						['txt1'],
						['txt2'],
						['txt4'],
						['licensekey']
					]
				}]
			}, {
				name: 'pageOverview',
				headerText: _('OX License Management'),
				helpText: _('<p>This module will assist you in configuring your Open-Xchange Customer Id and license key.</p><p>Your system has already been configured.</p>'),
				widgets: [{
					type: Text,
					name: 'contenttable',
					content: _('<p>...loading...</p>')
				},{
					type: Text,
					name: 'txt6',
					content: '<p>&nbsp;</p>'
				}],
				buttons: [{
					name: 'btnChange',
					label: _('Change settings'),
					callback: lang.hitch(this, function() {
						this._selectPage('pageAccount');
					})
				},{
					name: 'btnJumpToUpdater',
					label: _('Switch to Online Update module'),
					callback: function() {
						topic.publish("/umc/modules/open", "updater");
					}
				}],
				layout: [
					{
						label: _('OX License Management'),
						name: 'licenseSettings',
						layout: [['contenttable']]
					},
					['btnChange'],
					['btnJumpToUpdater'],
					['txt6']
				]
			}];
		},


		buildRendering: function() {
			this.inherited(arguments);
			this.standby(true);

			// get password widget and watch its value
			// ==> reset 'passwordExistsInBackend' to false if user changes value
			this.own(this.getWidget('pageAccount', 'password').watch('value', lang.hitch( this, function() {
				this.getWidget('pageAccount', 'password').set('passwordExistsInBackend', false);
			})));

			this.umcpCommand( 'oxldb/load' ).then( lang.hitch( this, function(response) {
				this._load(response);
				// first page is 'pageOverview' if dataset is complete and valid, otherwise 'pageAccount'
				if (! this.isComplete()) {
					this._selectPage('pageAccount');
				}
				domClass.toggle(this._pages.pageOverview._footerButtons.finish.domNode, 'dijitHidden', true);
			}));
		},

		_load: function(response) {
			this.set('_serverRole', response.result.serverRole);
			this.set('_ldapBase', response.result.ldapBase);
			this.set('_credentialsValid', response.result.credentialsValid);
			this.set('_currentLicensekey', response.result.currentLicensekey);

			this.getWidget('pageAccount', 'username').set('value', response.result.username);
			if (response.result.passwordExistsInBackend === true) {
				var w = this.getWidget('pageAccount', 'password');
				w.set('blockOnChange', true);              // do not emit onChange event
				w.set('value', this._dummyPassword);
				w.set('passwordExistsInBackend', true);
				w.set('blockOnChange', false);
			}

			// update overview
			var vtxt = lang.replace(_('<p><table cellspacing=6>') +
									_('<tr><td>Customer Id:</td><td>{username}</td></tr>') +
									_('<tr><td>Account status:</td><td>{accountvalid}</td></tr>') +
									_('<tr><td>License key:</td><td>{licensekey}</td></tr>') +
									_('<tr><td>LDAP base:</td><td>{ldapbase}</td></tr>') +
									_('</table></p>'), {
										username: response.result.username,
										accountvalid: response.result.credentialsValid ? _('credentials valid') : _('<b>credentials invalid</b>'),
										licensekey: response.result.currentLicensekey,
										ldapbase: response.result.ldapBase
									});
			this.getWidget('pageOverview', 'contenttable').set('content', vtxt);

			this.standby(false);
		},

		canCancel: function(pageName) {
			return ((pageName != 'pageOverview') && this.isComplete());
		},

		onCancel: function() {
			// switch to overview page
			this._selectPage('pageOverview');
			// load new settings
			this.umcpCommand( 'oxldb/load' ).then( lang.hitch( this, this._load ) );
		},

		hasNext: function(pageName) {
			return (pageName === 'pageAccount');
		},

		isComplete: function() {
			var val_username = this.getWidget('pageAccount', 'username').get('value');
			var val_password = this.getWidget('pageAccount', 'password').get('value');
			var val_licensekey = this.getWidget('pageLicense', 'licensekey').get('value');
			return (Boolean(val_username) && Boolean(val_password) && (Boolean(val_licensekey) || Boolean(this._currentLicensekey)));
		},

		next: function(currentPage) {
			var nextPage = this.inherited(arguments);

			// show pageOverview
			if (currentPage === null) {
				return 'pageOverview';
			}

			// next is only called on 'pageAccount', so check credentials and get list of licensekeys
			var passwordWidget = this.getWidget('pageAccount', 'password');
			var password = passwordWidget.get('value');
			var passwordExistsInBackend = (Boolean(passwordWidget.get('passwordExistsInBackend')) && (password == this._dummyPassword));
			var values = { 'username': this.getWidget('pageAccount', 'username').get('value'),
						   'password': password,
						   'passwordExistsInBackend': passwordExistsInBackend
						 };

 			this.umcpCommand('oxldb/licensekey/get', values).then( lang.hitch( this, function( response ) {
 				if ( !response.result.success ) {
 					dialog.alert(response.result.message, _('Error'));
 				} else {
 					dialog.notify( response.result.message );

 					this.set('_credentialsValid', response.result.success);
 					// save licensekey for next page
 					this.set('_licensekey', response.result.licensekey);
 					// update licensekey widget
					this.getWidget('pageLicense', 'licensekey').set('value', response.result.licensekey);

 					this._selectPage('pageLicense');
 				}
 				this.standby( false );
 			} ) );

 			// standby will be deactivated by callback of umcpCommand
 			this.standby(true, this.getWidget('pageAccount', 'waitmsg'));
			return 'pageAccount';
		},

		_selectPage: function(pageName) {
			this._updateButtons(pageName);
			this.selectChild(this._pages[pageName]);
			// workaround on pageOverview ==> hide "finish" button
			if (pageName == 'pageOverview') {
				domClass.toggle(this._pages[pageName]._footerButtons.finish.domNode, 'dijitHidden', true);
			}
		},

		hasPrevious: function(pageName) {
			return (pageName === 'pageLicense');
		},

		previous: function() {
			return 'pageAccount';
		},

		_password_lost: function(username) {
			var msg = _('<p>Please enter your current username and a new password (twice) to reset your current password. After pressing "Reset password" an email is sent to you. The mail contains a link that you have to open in your browser to confirm the password reset.</p>');
			var deferred = new Deferred();
			var pwLostDialog = null;
			var form = new Form({
				widgets: [{
					name: 'text',
					type: Text,
					content: msg
				}, {
					name: 'username',
					type: TextBox,
					label: _('Username'),
					value: username
				}, {
					name: 'password',
					type: PasswordBox,
					label: _('Password')
				},  {
					name: 'password2',
					type: PasswordBox,
					label: _('Password')
				}],
				buttons: [{
					name: 'cancel',
					label: _('Cancel'),
					callback: lang.hitch(this, function() {
						deferred.reject();
						this.standby(false);
						pwLostDialog.hide();
						pwLostDialog.destroyRecursive();
						form.destroyRecursive();
					})
				},{
					name: 'submit',
					label: _('Reset password'),
					callback: lang.hitch(this, function() {
						if (form.getWidget('password').get('value') != form.getWidget('password2').get('value')) {
							dialog.alert( _('The given passwords are not equal.'), _('Password mismatch') );
						} else if (form.getWidget('password').get('value').length === 0) {
							dialog.alert( _('The new password may not be empty.'), _('Password invalid') );
						} else {
							this.standby(false);
							deferred.resolve({
								username: form.getWidget('username').get('value'),
								password: form.getWidget('password').get('value')
							});
							pwLostDialog.hide();
							pwLostDialog.destroyRecursive();
							form.destroyRecursive();
						}
					})
				}],
				layout: [ 'text', 'username', 'password', 'password2' ]
			});
			pwLostDialog = new Dialog({
				title: _('Account data'),
				content: form,
				style: 'max-width: 400px;'
			});
			this.own(pwLostDialog);
			pwLostDialog.on('Hide', lang.hitch(this, function() {
				if (!deferred.isResolved()) {
					// user clicked the close button
					this.standby(false);
					deferred.reject();
				}
			}));
			pwLostDialog.show();
			return deferred;
		},

		_saveSettings: function() {
			this.standby(true, this.getWidget('pageAccount', 'savewaitmsg'));
			var password = this.getWidget('pageAccount', 'password').get('value');
			var licensekey = this.getWidget('pageLicense', 'licensekey').get('value');
			var passwordExistsInBackend = (this.getWidget('pageAccount', 'password').get('passwordExistsInBackend') && (password == this._dummyPassword));
			var values = { username: this.getWidget('pageAccount', 'username').get('value'),
						   password: password,
						   passwordExistsInBackend: passwordExistsInBackend,
						   licensekey: licensekey
						 };

			this.umcpCommand( 'oxldb/save', values ).then( lang.hitch( this, function( response ) {
				if ( !response.result.success ) {
					dialog.alert( response.result.message, _('Error') );
					this.standby( false );
				} else {
					if (response.result.note) {
						dialog.alert( _('System settings have been actualised successfully.<br /><br />%s', response.result.note), _('Success') );
					} else {
						dialog.alert( _('System settings have been actualised successfully.'), _('Success') );
					}
					// switch to overview page
					this._selectPage('pageOverview');
					// load new settings
					this.umcpCommand( 'oxldb/load' ).then( lang.hitch( this, this._load ) );
				}
			} ) );
		},

		onFinished: function( values ) {
			this._saveSettings();
		}
	});

	return declare("umc.modules.oxldb", [Module], {
		buildRendering: function() {
			this.inherited(arguments);
			this.oxldb = new oxldb({
				standby: lang.hitch(this, 'standby'),
				standbyDuring: lang.hitch(this, 'standbyDuring'),
				umcpCommand: lang.hitch(this, 'umcpCommand')
			});
			this.own(this.oxldb);
			this.addChild(this.oxldb);
		}
	});
});
